@@ -2,6 +2,9 @@
 //
 // SPDX-License-Identifier: AGPL-3.0-only
 
+//! Abstract Syntax Tree of the Nix expressions language.
+//!
+//! You most likely want to read the [AST] type first.
 use std::collections::LinkedList;
 use std::rc::Rc;
 
@@ -9,7 +12,6 @@ use santiago::lexer::Lexeme;
 use santiago::lexer::Position;
 
 /// Main type of an Abstract Syntax Tree.
-#[allow(clippy::manual_non_exhaustive)]
 #[derive(Clone, Debug, PartialEq)]
 pub enum AST {
     Assert {
@@ -27,8 +29,7 @@ pub enum AST {
         position: Position,
     },
     Function {
-        argument:   Option<String>,
-        arguments:  FunctionArguments,
+        argument:   FunctionArgument,
         definition: Box<AST>,
         position:   Position,
     },
@@ -65,8 +66,12 @@ pub enum AST {
         recursive: bool,
         position:  Position,
     },
+    Parentheses {
+        expression: Box<AST>,
+        position:   Position,
+    },
     Path {
-        parts:    LinkedList<StringPart>,
+        parts:    LinkedList<Part>,
         position: Position,
     },
     PropertyAccess {
@@ -79,7 +84,7 @@ pub enum AST {
         position: Position,
     },
     String {
-        parts:    LinkedList<StringPart>,
+        parts:    LinkedList<Part>,
         position: Position,
     },
     UnaryOperation {
@@ -103,75 +108,38 @@ pub enum AST {
 
     // Temporary containers
     #[doc(hidden)]
-    __,
-    #[doc(hidden)]
-    __Attribute(Attribute),
-    #[doc(hidden)]
-    __Attributes(LinkedList<Attribute>),
+    __Attributes(LinkedList<Part>),
     #[doc(hidden)]
     __AttributePath(AttributePath),
     #[doc(hidden)]
     __Bindings(LinkedList<Binding>),
     #[doc(hidden)]
-    __FunctionArgument(FunctionArgument),
+    __DestructuredArgument(DestructuredArgument),
     #[doc(hidden)]
-    __FunctionArguments(FunctionArguments),
-    #[doc(hidden)]
-    __StringParts(LinkedList<StringPart>),
+    __DestructuredArguments(DestructuredArguments),
     #[doc(hidden)]
     __Lexeme(Rc<Lexeme>),
-}
-
-impl AST {
-    pub fn position(&self) -> Position {
-        match &self {
-            AST::Assert { position, .. } => position.clone(),
-            AST::BinaryOperation { position, .. } => position.clone(),
-            AST::Float { position, .. } => position.clone(),
-            AST::Function { position, .. } => position.clone(),
-            AST::FunctionApplication { function, .. } => function.position(),
-            AST::HasProperty { position, .. } => position.clone(),
-            AST::IfThenElse { position, .. } => position.clone(),
-            AST::Int { position, .. } => position.clone(),
-            AST::LetIn { position, .. } => position.clone(),
-            AST::List { position, .. } => position.clone(),
-            AST::Map { position, .. } => position.clone(),
-            AST::Path { position, .. } => position.clone(),
-            AST::PropertyAccess { expression, .. } => expression.position(),
-            AST::SearchNixPath { position, .. } => position.clone(),
-            AST::String { position, .. } => position.clone(),
-            AST::UnaryOperation { position, .. } => position.clone(),
-            AST::Uri { position, .. } => position.clone(),
-            AST::Variable { position, .. } => position.clone(),
-            AST::With { position, .. } => position.clone(),
-            AST::__ => unreachable!(),
-            AST::__Attribute(_) => unreachable!(),
-            AST::__Attributes(_) => unreachable!(),
-            AST::__AttributePath(_) => unreachable!(),
-            AST::__Bindings(_) => unreachable!(),
-            AST::__FunctionArgument(_) => unreachable!(),
-            AST::__FunctionArguments(_) => unreachable!(),
-            AST::__StringParts(_) => unreachable!(),
-            AST::__Lexeme(_) => unreachable!(),
-        }
-    }
+    #[doc(hidden)]
+    __Part(Part),
+    #[doc(hidden)]
+    __Parts(LinkedList<Part>),
 }
 
 #[derive(Clone, Debug, PartialEq)]
-pub enum Attribute {
+pub enum Part {
     Raw { content: String, position: Position },
     Expression { expression: Box<AST> },
 }
 
 #[derive(Clone, Debug, PartialEq)]
 pub struct AttributePath {
-    pub attributes: LinkedList<Attribute>,
+    pub parts: LinkedList<Part>,
 }
 
 #[derive(Clone, Debug, PartialEq)]
 pub enum Binding {
-    KeyValue(AttributePath, Box<AST>),
-    Inherit(Option<Box<AST>>, LinkedList<Attribute>),
+    Binding { from: AttributePath, to: Box<AST> },
+    Inherit { from: Option<Box<AST>>, attributes: LinkedList<Part> },
 }
 
 #[derive(Clone, Debug, PartialEq)]
@@ -194,21 +162,34 @@ pub enum BinaryOperator {
 }
 
 #[derive(Clone, Debug, PartialEq)]
-pub struct FunctionArgument {
+pub enum DestructuredIdentifier {
+    None,
+    LeftAt(String),
+    RightAt(String),
+}
+
+#[derive(Clone, Debug, PartialEq)]
+pub struct DestructuredArgument {
     pub identifier: String,
     pub default:    Option<Box<AST>>,
 }
 
 #[derive(Clone, Debug, PartialEq)]
-pub struct FunctionArguments {
-    pub arguments: LinkedList<FunctionArgument>,
+pub struct DestructuredArguments {
+    pub arguments: LinkedList<DestructuredArgument>,
     pub ellipsis:  bool,
 }
 
 #[derive(Clone, Debug, PartialEq)]
-pub enum StringPart {
-    Raw { content: String },
-    Expression { expression: Box<AST> },
+pub enum FunctionArgument {
+    Destructured {
+        identifier: DestructuredIdentifier,
+        arguments:  LinkedList<DestructuredArgument>,
+        ellipsis:   bool,
+    },
+    Simple {
+        identifier: String,
+    },
 }
 
 #[derive(Clone, Debug, PartialEq)]
