@@ -3,6 +3,11 @@
 // SPDX-License-Identifier: AGPL-3.0-only
 
 use std::io::Read;
+use std::rc::Rc;
+
+use clap::ArgGroup;
+use nixel::cst::build_concrete_syntax_tree;
+use nixel::deps::santiago::lexer::Lexeme;
 
 fn main() -> Result<(), ()> {
     // Get CLI arguments
@@ -14,13 +19,8 @@ fn main() -> Result<(), ()> {
         Err(error) => return error,
     };
 
-    // HashSet with the names of the outputs we want to print
-    let outputs: std::collections::HashSet<&str> =
-        args.values_of("outputs").unwrap().collect();
-
     // Perform lexical analysis
     let lexing_rules = nixel::lexer::lexer_rules();
-
     let lexemes = match santiago::lexer::lex(&lexing_rules, &input) {
         Ok(lexemes) => lexemes,
         Err(error) => {
@@ -29,8 +29,15 @@ fn main() -> Result<(), ()> {
             return Err(());
         }
     };
+    let lexeme_no_trivia: Vec<Rc<Lexeme>> = lexemes
+        .iter()
+        .filter(|lexeme| &lexeme.kind != "COMMENT")
+        .filter(|lexeme| &lexeme.kind != "WS")
+        .cloned()
+        .collect();
 
-    if outputs.contains("lexemes") {
+    // Print lexical analysis results
+    if args.is_present("lex") {
         println!("Lexemes:");
         for lexeme in &lexemes {
             println!("  {lexeme}");
@@ -39,7 +46,8 @@ fn main() -> Result<(), ()> {
 
     // Perform parsing of the input
     let grammar = nixel::grammar::grammar();
-    let parse_tree = match santiago::parser::parse(&grammar, &lexemes) {
+    let parse_tree = match santiago::parser::parse(&grammar, &lexeme_no_trivia)
+    {
         Ok(mut parse_trees) => {
             if parse_trees.len() == 1 {
                 parse_trees.swap_remove(0)
@@ -56,7 +64,8 @@ fn main() -> Result<(), ()> {
         }
     };
 
-    if outputs.contains("parse-tree") {
+    // Print parsing results
+    if args.is_present("parse") {
         println!("Parse Tree:");
         println!("{parse_tree}");
     }
@@ -64,11 +73,21 @@ fn main() -> Result<(), ()> {
     // Generate the Abstract Syntax Tree
     let ast = parse_tree.as_abstract_syntax_tree();
 
-    if outputs.contains("ast") {
+    // Print the Abstract Syntax Tree
+    if args.is_present("ast") {
         println!("Abstract Syntax Tree:");
         println!("{ast:#?}");
     }
 
+    // Generate the Concrete Syntax Tree
+    let cst = build_concrete_syntax_tree(&ast, &lexemes);
+
+    // Print the Concrete Syntax Tree
+    if args.is_present("cst") {
+        println!("Concrete Syntax Tree:");
+        println!("{cst:#?}");
+    }
+
     Ok(())
 }
 
@@ -97,22 +116,46 @@ fn cli() -> clap::ArgMatches {
     clap::Command::new("NixEL")
         .about(
             "Lexer, Parser, Abstract Syntax Tree and Concrete Syntax Tree for \
-             the Nix Expressions Language.",
+             the Nix Expressions Language",
         )
-        .version("1.0.1")
+        .version("3.0.0")
         .arg(
             clap::Arg::new("path")
-                .help("File to process, or leave empty to process stdin."),
+                .help("File to process, or leave empty to process stdin"),
         )
         .arg(
-            clap::Arg::new("outputs")
-                .default_value("ast")
-                .help("What to print to stdout.")
-                .multiple_values(false)
-                .multiple_occurrences(true)
-                .possible_values(&["lexemes", "parse-tree", "ast"])
-                .short('o')
-                .takes_value(true),
+            clap::Arg::new("lex")
+                .help("Print lexical analysis results to stdout")
+                .long("lex")
+                .required(false)
+                .takes_value(false),
+        )
+        .arg(
+            clap::Arg::new("parse")
+                .help("Print parsing results to stdout")
+                .long("parse")
+                .required(false)
+                .takes_value(false),
+        )
+        .arg(
+            clap::Arg::new("ast")
+                .help("Build and print the Abstract Syntax Tree to stdout")
+                .long("ast")
+                .required(false)
+                .takes_value(false),
+        )
+        .arg(
+            clap::Arg::new("cst")
+                .help("Build and print the Concrete Syntax Tree to stdout")
+                .long("cst")
+                .required(false)
+                .takes_value(false),
+        )
+        .group(
+            ArgGroup::new("outputs")
+                .args(&["lex", "parse", "ast", "cst"])
+                .required(true)
+                .multiple(true),
         )
         .term_width(80)
         .get_matches()
