@@ -2,19 +2,22 @@
 //
 // SPDX-License-Identifier: AGPL-3.0-only
 
+//! Grammar of the Nix expressions language.
+
 use std::collections::LinkedList;
 
 use santiago::grammar::Associativity;
 use santiago::grammar::Grammar;
 use santiago::lexer::Position;
 
-use crate::ast::Attribute;
 use crate::ast::AttributePath;
 use crate::ast::BinaryOperator;
 use crate::ast::Binding;
+use crate::ast::DestructuredArgument;
+use crate::ast::DestructuredArguments;
+use crate::ast::DestructuredIdentifier;
 use crate::ast::FunctionArgument;
-use crate::ast::FunctionArguments;
-use crate::ast::StringPart;
+use crate::ast::Part;
 use crate::ast::UnaryOperator;
 use crate::ast::AST;
 
@@ -25,10 +28,8 @@ pub fn grammar() -> Grammar<AST> {
         "expr_function" => rules "ID" ":" "expr_function"
             => |mut asts| match asts.swap_remove(0) {
                 AST::__Lexeme(lexeme) => AST::Function {
-                    argument: Some(lexeme.raw.clone()),
-                    arguments: FunctionArguments {
-                        arguments: LinkedList::new(),
-                        ellipsis: false,
+                    argument: FunctionArgument::Simple {
+                        identifier: lexeme.raw.clone(),
                     },
                     definition: Box::new(asts.swap_remove(0)),
                     position: lexeme.position.clone(),
@@ -38,10 +39,17 @@ pub fn grammar() -> Grammar<AST> {
         "expr_function" => rules "{" "formals" "}" ":" "expr_function"
             => |mut asts| match asts.swap_remove(0) {
                 AST::__Lexeme(lexeme) => AST::Function {
-                    argument: None,
-                    arguments: match asts.swap_remove(1) {
-                        AST::__FunctionArguments(function_arguments) => function_arguments,
-                        _ => unreachable!(),
+                    argument: {
+                        let formals = match asts.swap_remove(1) {
+                            AST::__DestructuredArguments(formals) => formals,
+                            _ => unreachable!(),
+                        };
+
+                        FunctionArgument::Destructured {
+                            identifier: DestructuredIdentifier::None,
+                            arguments: formals.arguments,
+                            ellipsis: formals.ellipsis,
+                        }
                     },
                     definition: Box::new(asts.swap_remove(0)),
                     position: lexeme.position.clone(),
@@ -51,13 +59,23 @@ pub fn grammar() -> Grammar<AST> {
         "expr_function" => rules "{" "formals" "}" "@" "ID" ":" "expr_function"
             => |mut asts| match asts.swap_remove(0) {
                 AST::__Lexeme(lexeme) => AST::Function {
-                    argument: Some(match asts.swap_remove(4) {
-                        AST::__Lexeme(lexeme) => lexeme.raw.clone(),
-                        _ => unreachable!(),
-                    }),
-                    arguments: match asts.swap_remove(1) {
-                        AST::__FunctionArguments(function_arguments) => function_arguments,
-                        _ => unreachable!(),
+                    argument: {
+                        let formals = match asts.swap_remove(1) {
+                            AST::__DestructuredArguments(formals) => formals,
+                            _ => unreachable!(),
+                        };
+                        let identifier = match asts.swap_remove(4) {
+                            AST::__Lexeme(lexeme) => DestructuredIdentifier::RightAt(
+                                lexeme.raw.clone()
+                            ),
+                            _ => unreachable!(),
+                        };
+
+                        FunctionArgument::Destructured {
+                            identifier,
+                            arguments: formals.arguments,
+                            ellipsis: formals.ellipsis,
+                        }
                     },
                     definition: Box::new(asts.swap_remove(0)),
                     position: lexeme.position.clone(),
@@ -67,10 +85,20 @@ pub fn grammar() -> Grammar<AST> {
         "expr_function" => rules "ID" "@" "{" "formals" "}" ":" "expr_function"
             => |mut asts| match asts.swap_remove(0) {
                 AST::__Lexeme(lexeme) => AST::Function {
-                    argument: Some(lexeme.raw.clone()),
-                    arguments: match asts.swap_remove(3) {
-                        AST::__FunctionArguments(function_arguments) => function_arguments,
-                        _ => unreachable!(),
+                    argument: {
+                        let formals = match asts.swap_remove(3) {
+                            AST::__DestructuredArguments(formals) => formals,
+                            _ => unreachable!(),
+                        };
+                        let identifier = DestructuredIdentifier::LeftAt(
+                            lexeme.raw.clone()
+                        );
+
+                        FunctionArgument::Destructured {
+                            identifier,
+                            arguments: formals.arguments,
+                            ellipsis: formals.ellipsis,
+                        }
                     },
                     definition: Box::new(asts.swap_remove(0)),
                     position: lexeme.position.clone(),
@@ -425,7 +453,7 @@ pub fn grammar() -> Grammar<AST> {
             };
         "expr_simple" => rules "\"" "string_parts" "\""
             => |mut asts| match asts.swap_remove(1) {
-                AST::__StringParts(parts) => AST::String {
+                AST::__Parts(parts) => AST::String {
                     parts,
                     position: match asts.swap_remove(0) {
                         AST::__Lexeme(lexeme) => lexeme.position.clone(),
@@ -436,7 +464,7 @@ pub fn grammar() -> Grammar<AST> {
             };
         "expr_simple" => rules "IND_STRING_OPEN" "ind_string_parts" "IND_STRING_CLOSE"
             => |mut asts| match asts.swap_remove(1) {
-                AST::__StringParts(parts) => AST::String {
+                AST::__Parts(parts) => AST::String {
                     parts: strip_indentation(parts),
                     position: match asts.swap_remove(0) {
                         AST::__Lexeme(lexeme) => lexeme.position.clone(),
@@ -457,7 +485,7 @@ pub fn grammar() -> Grammar<AST> {
                     _ => unreachable!(),
                 };
                 let mut string_parts_interpolated = match asts.swap_remove(1) {
-                    AST::__StringParts(string_parts) => string_parts,
+                    AST::__Parts(parts) => parts,
                     _ => unreachable!(),
                 };
 
@@ -484,7 +512,13 @@ pub fn grammar() -> Grammar<AST> {
                 _ => unreachable!(),
             };
         "expr_simple" => rules "(" "expr" ")"
-            => |mut asts| asts.swap_remove(1);
+            => |mut asts| AST::Parentheses {
+                expression: Box::new(asts.swap_remove(1)),
+                position: match asts.swap_remove(0) {
+                    AST::__Lexeme(lexeme) => lexeme.position.clone(),
+                    _ => unreachable!(),
+                },
+            };
         // "expr_simple" => rules "LET" "{" "binds" "}"
         "expr_simple" => rules "REC" "{" "binds" "}"
             => |mut asts| {
@@ -536,30 +570,32 @@ pub fn grammar() -> Grammar<AST> {
             };
 
         "string_parts" => rules "STR"
-            => |mut asts| AST::__StringParts(
+            => |mut asts| AST::__Parts(
                 LinkedList::from([
-                    StringPart::Raw {
-                        content: match asts.swap_remove(0) {
-                            AST::__Lexeme(lexeme) => lexeme.raw.clone(),
-                            _ => unreachable!(),
-                        }
-                    }
+                    match asts.swap_remove(0) {
+                        AST::__Lexeme(lexeme) => Part::Raw {
+                            content: lexeme.raw.clone(),
+                            position: lexeme.position.clone(),
+                        },
+                        _ => unreachable!(),
+                    },
                 ]),
             );
         "string_parts" => rules "string_parts_interpolated"
             => |mut asts| asts.swap_remove(0);
         "string_parts" => empty
-            => |_| AST::__StringParts(LinkedList::new());
+            => |_| AST::__Parts(LinkedList::new());
 
         "string_parts_interpolated" => rules "string_parts_interpolated" "STR"
             => |mut asts| {
                 let mut string_parts_interpolated = asts.swap_remove(0);
 
                 match &mut string_parts_interpolated {
-                    AST::__StringParts(string_parts) => {
-                        string_parts.push_back(match asts.swap_remove(0) {
-                            AST::__Lexeme(lexeme) => StringPart::Raw {
+                    AST::__Parts(parts) => {
+                        parts.push_back(match asts.swap_remove(0) {
+                            AST::__Lexeme(lexeme) => Part::Raw {
                                 content: lexeme.raw.clone(),
+                                position: lexeme.position.clone(),
                             },
                             _ => unreachable!(),
                         });
@@ -574,8 +610,8 @@ pub fn grammar() -> Grammar<AST> {
                 let mut string_parts_interpolated = asts.swap_remove(0);
 
                 match &mut string_parts_interpolated {
-                    AST::__StringParts(string_parts) => {
-                        string_parts.push_back(StringPart::Expression {
+                    AST::__Parts(parts) => {
+                        parts.push_back(Part::Expression {
                             expression: Box::new(asts.swap_remove(2)),
                         });
                     }
@@ -585,20 +621,21 @@ pub fn grammar() -> Grammar<AST> {
                 string_parts_interpolated
             };
         "string_parts_interpolated" => rules "DOLLAR_CURLY" "expr" "}"
-            => |mut asts| AST::__StringParts(LinkedList::from([
-                StringPart::Expression {
+            => |mut asts| AST::__Parts(LinkedList::from([
+                Part::Expression {
                     expression: Box::new(asts.swap_remove(1)),
                 }
             ]));
         "string_parts_interpolated" => rules "STR" "DOLLAR_CURLY" "expr" "}"
-            => |mut asts| AST::__StringParts(LinkedList::from([
+            => |mut asts| AST::__Parts(LinkedList::from([
                 match asts.swap_remove(0) {
-                    AST::__Lexeme(lexeme) => StringPart::Raw {
+                    AST::__Lexeme(lexeme) => Part::Raw {
                         content: lexeme.raw.clone(),
+                        position: lexeme.position.clone(),
                     },
                     _ => unreachable!(),
                 },
-                StringPart::Expression {
+                Part::Expression {
                     expression: Box::new(asts.swap_remove(2)),
                 },
             ]));
@@ -607,8 +644,9 @@ pub fn grammar() -> Grammar<AST> {
             => |mut asts| match asts.swap_remove(0) {
                 AST::__Lexeme(lexeme) => AST::Path {
                     parts: LinkedList::from([
-                        StringPart::Raw {
+                        Part::Raw {
                             content: lexeme.raw.clone(),
+                            position: lexeme.position.clone(),
                         },
                     ]),
                     position: lexeme.position.clone(),
@@ -619,8 +657,9 @@ pub fn grammar() -> Grammar<AST> {
             => |mut asts| match asts.swap_remove(0) {
                 AST::__Lexeme(lexeme) => AST::Path {
                     parts: LinkedList::from([
-                        StringPart::Raw {
+                        Part::Raw {
                             content: lexeme.raw.clone(),
+                            position: lexeme.position.clone(),
                         },
                     ]),
                     position: lexeme.position.clone(),
@@ -633,10 +672,11 @@ pub fn grammar() -> Grammar<AST> {
                 let mut ind_string_parts = asts.swap_remove(0);
 
                 match &mut ind_string_parts {
-                    AST::__StringParts(parts) => match asts.swap_remove(0) {
+                    AST::__Parts(parts) => match asts.swap_remove(0) {
                         AST::__Lexeme(lexeme) => {
-                            parts.push_back(StringPart::Raw {
+                            parts.push_back(Part::Raw {
                                 content: lexeme.raw.clone(),
+                                position: lexeme.position.clone(),
                             });
                         },
                         _ => unreachable!(),
@@ -650,8 +690,8 @@ pub fn grammar() -> Grammar<AST> {
                 let mut ind_string_parts = asts.swap_remove(0);
 
                 match &mut ind_string_parts {
-                    AST::__StringParts(parts) => {
-                        parts.push_back(StringPart::Expression {
+                    AST::__Parts(parts) => {
+                        parts.push_back(Part::Expression {
                             expression: Box::new(asts.swap_remove(2)),
                         });
                     }
@@ -660,7 +700,7 @@ pub fn grammar() -> Grammar<AST> {
                 ind_string_parts
             };
         "ind_string_parts" => empty
-            => |_| AST::__StringParts(LinkedList::new());
+            => |_| AST::__Parts(LinkedList::new());
 
         "binds" => rules "binds" "attrpath" "=" "expr" ";"
             => |mut asts| {
@@ -668,13 +708,13 @@ pub fn grammar() -> Grammar<AST> {
 
                 match &mut binds {
                     AST::__Bindings(bindings) => {
-                        bindings.push_back(Binding::KeyValue(
-                            match asts.swap_remove(1) {
+                        bindings.push_back(Binding::Binding {
+                            from: match asts.swap_remove(1) {
                                 AST::__AttributePath(attribute_path) => attribute_path,
                                 _ => unreachable!(),
                             },
-                            Box::new(asts.swap_remove(1)),
-                        ));
+                            to: Box::new(asts.swap_remove(1)),
+                        });
                     },
                     _ => unreachable!(),
                 }
@@ -689,10 +729,10 @@ pub fn grammar() -> Grammar<AST> {
                     AST::__Bindings(bindings) => {
                         match asts.swap_remove(2) {
                             AST::__Attributes(attributes) => {
-                                bindings.push_back(Binding::Inherit(
-                                    None,
+                                bindings.push_back(Binding::Inherit {
+                                    from: None,
                                     attributes,
-                                ));
+                                });
                             },
                             _ => unreachable!(),
                         }
@@ -710,10 +750,10 @@ pub fn grammar() -> Grammar<AST> {
                     AST::__Bindings(bindings) => {
                         match asts.swap_remove(5) {
                             AST::__Attributes(attributes) => {
-                                bindings.push_back(Binding::Inherit(
-                                    Some(Box::new(asts.swap_remove(3))),
+                                bindings.push_back(Binding::Inherit {
+                                    from: Some(Box::new(asts.swap_remove(3))),
                                     attributes,
-                                ));
+                                });
                             },
                             _ => unreachable!(),
                         }
@@ -728,35 +768,35 @@ pub fn grammar() -> Grammar<AST> {
 
         "attrs" => rules "attrs" "attr"
             => |mut asts| {
-                let mut attrpath = asts.swap_remove(0);
+                let mut attrs = asts.swap_remove(0);
 
-                match &mut attrpath {
+                match &mut attrs {
                     AST::__Attributes(attributes) => {
                         attributes.push_back(match asts.swap_remove(0) {
-                            AST::__Attribute(attribute) => attribute,
+                            AST::__Part(attribute) => attribute,
                             _ => unreachable!(),
                         });
                     },
                     _ => unreachable!(),
                 }
 
-                attrpath
+                attrs
             };
         "attrs" => rules "attrs" "string_attr"
             => |mut asts| {
-                let mut attrpath = asts.swap_remove(0);
+                let mut attrs = asts.swap_remove(0);
 
-                match &mut attrpath {
+                match &mut attrs {
                     AST::__Attributes(attributes) => {
                         attributes.push_back(match asts.swap_remove(0) {
-                            AST::__Attribute(attribute) => attribute,
+                            AST::__Part(attribute) => attribute,
                             _ => unreachable!(),
                         });
                     },
                     _ => unreachable!(),
                 }
 
-                attrpath
+                attrs
             };
         "attrs" => empty
             => |_| AST::__Attributes(LinkedList::new());
@@ -767,9 +807,9 @@ pub fn grammar() -> Grammar<AST> {
 
                 match &mut attrpath {
                     AST::__AttributePath(attribute_path) => {
-                        attribute_path.attributes.push_back(
+                        attribute_path.parts.push_back(
                             match asts.swap_remove(0) {
-                                AST::__Attribute(attribute) => attribute,
+                                AST::__Part(attribute) => attribute,
                                 _ => unreachable!(),
                             },
                         );
@@ -785,9 +825,9 @@ pub fn grammar() -> Grammar<AST> {
 
                 match &mut attrpath {
                     AST::__AttributePath(attribute_path) => {
-                        attribute_path.attributes.push_back(
+                        attribute_path.parts.push_back(
                             match asts.swap_remove(0) {
-                                AST::__Attribute(attribute) => attribute,
+                                AST::__Part(attribute) => attribute,
                                 _ => unreachable!(),
                             },
                         );
@@ -799,18 +839,18 @@ pub fn grammar() -> Grammar<AST> {
             };
         "attrpath" => rules "attr"
             => |mut asts| AST::__AttributePath(AttributePath {
-                attributes: LinkedList::from([
+                parts: LinkedList::from([
                     (match asts.swap_remove(0) {
-                        AST::__Attribute(attribute) => attribute,
+                        AST::__Part(attribute) => attribute,
                         _ => unreachable!(),
                     })
                 ]),
             });
         "attrpath" => rules "string_attr"
             => |mut asts| AST::__AttributePath(AttributePath {
-                attributes: LinkedList::from([
+                parts: LinkedList::from([
                     (match asts.swap_remove(0) {
-                        AST::__Attribute(attribute) => attribute,
+                        AST::__Part(attribute) => attribute,
                         _ => unreachable!(),
                     })
                 ]),
@@ -818,7 +858,7 @@ pub fn grammar() -> Grammar<AST> {
         "attr" => rules "ID"
             => |mut asts| match asts.swap_remove(0) {
                 AST::__Lexeme(lexeme) => {
-                    AST::__Attribute(Attribute::Raw {
+                    AST::__Part(Part::Raw {
                         content: lexeme.raw.clone(),
                         position: lexeme.position.clone(),
                     })
@@ -828,7 +868,7 @@ pub fn grammar() -> Grammar<AST> {
         "attr" => rules "OR_KW"
             => |mut asts| match asts.swap_remove(0) {
                 AST::__Lexeme(lexeme) => {
-                    AST::__Attribute(Attribute::Raw {
+                    AST::__Part(Part::Raw {
                         content: lexeme.raw.clone(),
                         position: lexeme.position.clone(),
                     })
@@ -837,11 +877,20 @@ pub fn grammar() -> Grammar<AST> {
             };
 
         "string_attr" => rules "\"" "string_parts" "\""
-            => |mut asts| AST::__Attribute(Attribute::Expression {
-                expression: Box::new(asts.swap_remove(1)),
+            => |mut asts| AST::__Part(Part::Expression {
+                expression: Box::new(match asts.swap_remove(1) {
+                    AST::__Parts(parts) => AST::String {
+                        parts,
+                        position: match asts.swap_remove(0) {
+                            AST::__Lexeme(lexeme) => lexeme.position.clone(),
+                            _ => unreachable!(),
+                        },
+                    },
+                    _ => unreachable!(),
+                }),
             });
         "string_attr" => rules "DOLLAR_CURLY" "expr" "}"
-            => |mut asts| AST::__Attribute(Attribute::Expression {
+            => |mut asts| AST::__Part(Part::Expression {
                 expression: Box::new(asts.swap_remove(1)),
             });
 
@@ -871,13 +920,13 @@ pub fn grammar() -> Grammar<AST> {
             => |mut asts| {
                 let mut formals = asts.swap_remove(2);
                 let formal = match asts.swap_remove(0) {
-                    AST::__FunctionArgument(function_argument) => function_argument,
+                    AST::__DestructuredArgument(formal) => formal,
                     _ => unreachable!(),
                 };
 
                 match &mut formals {
-                    AST::__FunctionArguments(function_arguments) => {
-                        function_arguments.arguments.push_back(formal);
+                    AST::__DestructuredArguments(formals) => {
+                        formals.arguments.push_front(formal);
                     },
                     _ => unreachable!(),
                 }
@@ -887,25 +936,31 @@ pub fn grammar() -> Grammar<AST> {
         "formals" => rules "formal"
             => |mut asts| {
                 let formal = match asts.swap_remove(0) {
-                    AST::__FunctionArgument(function_argument) => function_argument,
+                    AST::__DestructuredArgument(formal) => formal,
                     _ => unreachable!(),
                 };
 
-                AST::__FunctionArguments(FunctionArguments {
-                    arguments: LinkedList::from([formal]),
-                    ellipsis: false,
-                })
+                AST::__DestructuredArguments(
+                    DestructuredArguments {
+                        arguments: LinkedList::from([formal]),
+                        ellipsis: false,
+                    },
+                )
             };
         "formals" => rules "ELLIPSIS"
-            => |_| AST::__FunctionArguments(FunctionArguments {
-                arguments: LinkedList::new(),
-                ellipsis: true,
-            });
+            => |_| AST::__DestructuredArguments(
+                DestructuredArguments {
+                    arguments: LinkedList::new(),
+                    ellipsis: true,
+                },
+            );
         "formals" => empty
-            => |_| AST::__FunctionArguments(FunctionArguments {
-                arguments: LinkedList::new(),
-                ellipsis: false,
-            });
+            => |_| AST::__DestructuredArguments(
+                DestructuredArguments {
+                    arguments: LinkedList::new(),
+                    ellipsis: false,
+                },
+            );
 
         "formal" => rules "ID"
             => |mut asts| {
@@ -914,7 +969,7 @@ pub fn grammar() -> Grammar<AST> {
                     _ => unreachable!(),
                 };
 
-                AST::__FunctionArgument(FunctionArgument {
+                AST::__DestructuredArgument(DestructuredArgument {
                     identifier: id.raw.clone(),
                     default: None,
                 })
@@ -926,7 +981,7 @@ pub fn grammar() -> Grammar<AST> {
                     _ => unreachable!(),
                 };
 
-                AST::__FunctionArgument(FunctionArgument {
+                AST::__DestructuredArgument(DestructuredArgument {
                     identifier: id.raw.clone(),
                     default: Some(Box::new(asts.swap_remove(0))),
                 })
@@ -1059,16 +1114,14 @@ pub fn grammar() -> Grammar<AST> {
     )
 }
 
-fn strip_indentation(
-    mut parts: LinkedList<StringPart>,
-) -> LinkedList<StringPart> {
+fn strip_indentation(mut parts: LinkedList<Part>) -> LinkedList<Part> {
     let mut at_start_of_line = true;
     let mut min_indent: usize = usize::MAX;
     let mut cur_indent: usize = 0;
 
     // Compute the min indentation
     for part in &parts {
-        if let StringPart::Raw { content, .. } = part {
+        if let Part::Raw { content, .. } = part {
             for char in content.chars() {
                 if at_start_of_line {
                     match char {
@@ -1098,7 +1151,7 @@ fn strip_indentation(
     let parts_count = parts.len();
     for part in parts.iter_mut() {
         match part {
-            StringPart::Raw { content, .. } => {
+            Part::Raw { content, .. } => {
                 *content = content
                     .chars()
                     .filter(|char| {
@@ -1138,7 +1191,7 @@ fn strip_indentation(
                     }
                 }
             }
-            StringPart::Expression { .. } => {
+            Part::Expression { .. } => {
                 at_start_of_line = false;
                 cur_dropped = 0;
             }
