@@ -2,7 +2,8 @@
 //
 // SPDX-License-Identifier: AGPL-3.0-only
 
-//! Lexical Analysis facilities.
+//! Lexer of the Nix expressions language.
+
 use santiago::lexer::LexerRules;
 
 santiago::def!(ANY, r"(?:.|\n)");
@@ -210,11 +211,11 @@ pub fn lexer_rules() -> LexerRules {
             };
         "DEFAULT" | "URI" = pattern URI!();
         "DEFAULT" | "WS" = pattern r"[ \t\r\n]+"
-            => |lexer| lexer.skip();
+            => |lexer| lexer.take();
         "DEFAULT" | "COMMENT" = pattern r"\#[^\r\n]*"
-            => |lexer| lexer.skip();
+            => |lexer| lexer.take();
         "DEFAULT" | "COMMENT" = pattern r"/\*([^*]|\*+[^*/])*\*+/"
-            => |lexer| lexer.skip();
+            => |lexer| lexer.take();
         //
         "DEFAULT" | "*" = string "*";
         "DEFAULT" | ":" = string ":";
